from pydantic import BaseModel

from .account import Account
from .jar import Jar


class Client(BaseModel):
    """Інформація про клієнта"""
    id: str
    """Ідентифікатор клієнта (збігається з id для send.monobank.ua)"""
    name: str
    """Ім'я клієнта"""
    webhook_url: str
    """URL для надсилання подій по зміні балансу рахунку"""
    permissions: str
    """Перелік прав, які надає сервіс (1 літера на 1 permission)."""
    accounts: list[Account]
    """Перелік доступних рахунків"""
    jars: list[Jar] = []
    """Перелік банок"""

    class Config:
        fields = {
            'client_id': 'clientId',
            'web_hook_url': 'webHookUrl',
        }



from .currency import Currency
from .client_info import Client
from .account import Account
from .jar import Jar
from .statement_item import Statement
from .webhook_data import WebhookData

__all__ = [
    'Currency',
    'Client',
    'Account',
    'Jar',
    'Statement',
    'WebhookData'
]



from datetime import datetime
from typing import Optional

import pytz
from pydantic import BaseModel, validator


class Statement(BaseModel):
    id: str
    """Унікальний id транзакції"""
    time: datetime
    """Час транзакції в секундах в форматі UTC time"""
    description: str
    """Опис транзакцій"""
    mcc: int
    """Код типу транзакції (Merchant Category Code), відповідно ISO 18245"""
    original_mcc: int
    """Оригінальний код типу транзакції (Merchant Category Code), відповідно ISO 18245"""
    hold: bool
    """Статус блокування суми (детальніше https://en.wikipedia.org/wiki/Authorization_hold)"""
    amount: float
    """Сума у валюті рахунку"""
    operation_amount: float
    """Сума у валюті транзакції"""
    currency_code: int
    """Код валюти рахунку відповідно ISO 4217"""
    commission_rate: float
    """Розмір комісії"""
    cashback_amount: float
    """Розмір кешбеку"""
    balance: float
    """Баланс рахунку в мінімальних одиницях валюти (копійках, центах)"""
    comment: Optional[str]
    """Коментар до переказу, уведений користувачем. Якщо не вказаний, поле буде відсутнім"""
    receipt_id: Optional[str]
    """Номер квитанції для check.gov.ua. Поле може бути відсутнім"""
    invoice_id: Optional[str]
    """Номер квитанції ФОПа, приходить у випадку якщо це операція із зарахуванням коштів"""
    counter_edrpou: Optional[str]
    """ЄДРПОУ контрагента, присутній лише для елементів виписки рахунків ФОП"""
    counter_iban: Optional[str]
    """IBAN контрагента, присутній лише для елементів виписки рахунків ФОП"""
    counter_name: Optional[str]
    """Найменування контрагента"""

    @validator('amount', 'operation_amount', 'cashback_amount', 'commission_rate', 'cashback_amount', 'balance',
               pre=True, allow_reuse=True)
    def _convert_from_integer_to_currency_sum(cls, value: int):
        return value / 100

    def get_time_for_timezone(self, timezone: str = 'Europe/Kyiv'):
        """
        The get_time_for_timezone function returns the current time in a given timezone.

        :param self: Represent the instance of the class
        :param timezone: str: Set the default timezone
        :return: The current time in the specified time zone
        """
        return self.time.astimezone(pytz.timezone(timezone))

    class Config:
        fields = {
            'original_mcc': 'originalMcc',
            'operation_amount': 'operationAmount',
            'currency_code': 'currencyCode',
            'commission_rate': 'commissionRate',
            'cashback_amount': 'cashbackAmount',
            'receipt_id': 'receiptId',
            'invoice_id': 'invoiceId',
            'counter_edrpou': 'counterEdrpou',
            'counter_iban': 'counterIban',
            'counter_name': 'counterName',
        }



from datetime import datetime

import pytz
from pydantic import BaseModel


class Currency(BaseModel):
    """Інформація про курс валют"""

    currency_code_a: str
    """Код валюти рахунку відповідно ISO 4217"""
    currency_code_b: str
    """Код валюти рахунку відповідно ISO 4217"""
    date: datetime
    """Час курсу в форматі UTC time"""
    rate_sell: float
    rate_buy: float
    rate_cross: float

    def get_time_for_timezone(self, timezone: str = 'Europe/Kyiv'):
        """
        The get_time_for_timezone function returns the current time in a given timezone.

        :param self: Represent the instance of the class
        :param timezone: str: Set the default timezone
        :return: The current time in the specified time zone
        """
        return self.date.astimezone(pytz.timezone(timezone))

    class Config:
        fields = {
            'currency_code_a': 'currencyCodeA',
            'currency_code_b': 'currencyCodeB',
            'rate_sell': 'rateSell',
            'rate_buy': 'rateBuy',
            'rate_cross': 'rateCross',
        }



from .exceptions import (
    Unauthorized,
    RetryAfter,
    InvalidAccount,
    PeriodError,
    InvalidToken,
    WebhookUrlError,
    NetworkError,
    ValidationError,
)


__all__ = [
    'Unauthorized',
    'RetryAfter',
    'InvalidAccount',
    'PeriodError',
    'InvalidToken',
    'WebhookUrlError',
    'NetworkError',
    'ValidationError'
]



import json
import ssl
import certifi
from typing import Optional
from http import HTTPMethod  # noqa

import aiohttp
from aiohttp import hdrs

from . import api
from .api import MonobankAPIServer, MONOBANK_PRODUCTION


class BaseMonobank:
    def __init__(
            self,
            token: str,
            validate_token: Optional[bool] = True,
            connections_limit: Optional[int] = None,
            server: MonobankAPIServer = MONOBANK_PRODUCTION
    ) -> None:
        """
        Create Monobank API token from https://api.monobank.ua/

        :param token: str: token from https://api.monobank.ua/
        :param server: MonobankAPIServer: Monobank API Server endpoint.
        :raise aiomonobank.utils.exceptions.ValidationError: when the token is invalid
        """
        # Authentication
        if validate_token:
            api.check_token(token)

        self._token = token
        self.server = server

        # aiohttp main session
        ssl_context = ssl.create_default_context(cafile=certifi.where())

        self._session: Optional[aiohttp.ClientSession] = None
        self._connector_class: aiohttp.TCPConnector = aiohttp.TCPConnector  # noqa
        self._connector_init = dict(limit=connections_limit, ssl=ssl_context)

    async def get_new_session(self) -> aiohttp.ClientSession:
        """
        The get_new_session function is a coroutine that returns an aiohttp.ClientSession object with the following properties:
            - The headers are set to accept JSON and include the token for authentication.
            - The connector is set to use our custom connector class, which we will define in just a moment.
            - json_serialize is set to use Python's built-in json module.

        :param self: Access the attributes and methods of the class
        :return: A new aiohttp
        """
        return aiohttp.ClientSession(
            headers={
                hdrs.ACCEPT: "application/json",
                "X-Token": self._token
            },
            connector=self._connector_class(**self._connector_init),
            json_serialize=json.dumps
        )

    async def get_session(self) -> aiohttp.ClientSession:
        """
        The get_session function is a coroutine that returns an aiohttp.ClientSession object.

        :param self: Refer to the current object
        :return: A client session object
        """
        if self._session is None or self._session.closed:
            self._session = await self.get_new_session()

        if not self._session._loop.is_running():  # noqa
            await self._session.close()
            self._session = await self.get_new_session()

        return self._session

    async def close(self):
        """
        Close all client sessions
        """
        if self._session:
            await self._session.close()

    async def request(self,
                      http_method: HTTPMethod,
                      path: str,
                      **kwargs) -> dict:
        """
        The request function is a wrapper around the make_request function in
        the api module. It takes an http method, a path, and any other arguments
        that are required for making the request (such as data or params). The
        request function will then return the response from make_request.

        :param self: Represent the instance of the class
        :param http_method: HTTPMethod: Specify the type of request that is being made
        :param path: str: Specify the path of the request
        :param **kwargs: Pass in any number of keyword arguments
        :return: A dictionary of data
        """
        return await api.make_request(
            session=await self.get_session(),
            server=self.server,
            http_method=http_method,
            api_path=path,
            **kwargs
        )

    async def __aenter__(self):
        await self.get_session()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.close()




import logging
from dataclasses import dataclass
from http import HTTPStatus, HTTPMethod  # noqa
import json
from urllib.parse import urljoin

import aiohttp

from .utils import exceptions

# Main aiomonobank logger
log = logging.getLogger('aiomonobank')


@dataclass(frozen=True)
class MonobankAPIServer:
    """
    Base config for API Endpoints
    """
    base_url: str

    def api_url(self, api_path: str) -> str:
        """
        The api_url function takes a string as an argument and returns a string.
        The returned string is the base url for the api with the api_path appended to it.

        :param self: Represent the instance of the class
        :param api_path: str: Specify the path of the api
        :return: The base url with the api_path appended to it
        """
        return urljoin(self.base_url, api_path)

    @classmethod
    def from_base(cls, base: str) -> 'MonobankAPIServer':
        """
        The from_base function is a class method that allows you to create an instance of the MonobankAPIServer
        class by providing only the base URL for the API server. This is useful if you want to use a different
        Monobank API server than what's provided in this library, or if you're using your own local development
        server. The from_base function will automatically add /{{api_path}} to whatever base URL string it receives,
        so that all requests are properly routed.

        :param cls: Refer to the class itself
        :param base: str: Specify the base url of the api
        :return: A monobank api server object with the base attribute set to the value of
        """
        return cls(
            base_url=base.rstrip("/"),
        )


MONOBANK_PRODUCTION = MonobankAPIServer.from_base("https://api.monobank.ua")


def check_token(token: str) -> bool:
    """
    The check_token function checks if the token is valid.

    :param token: str: Specify the type of the parameter
    :return: True if the token is valid and raises an exception otherwise
    """
    if not isinstance(token, str):
        raise exceptions.ValidationError(f"Token is invalid! It must be 'str' type instead of {type(token)} type.")

    if any(x.isspace() for x in token):
        raise exceptions.ValidationError("Token is invalid! It can't contains spaces.")

    return True


def check_result(api_path: str, content_type: str, status_code: int, body: str) -> dict:
    """
    The check_result function is used to check the response from Monobank API.
    It checks if the content type of the response is application/json, and if it's not - raises a NetworkError exception.
    If it's OK, then we try to parse JSON from body of our request and store it in result_json variable.
    Then we check status code: if everything is OK (200), then return result_json; otherwise raise an appropriate exception.

    :param api_path: str: Specify the path to the api method
    :param content_type: str: Check the content type of the response
    :param status_code: int: Check the status code of the response
    :param body: str: Pass the body of the response from monobank api
    :return: The dictionary with the following keys if the status code is 200
    """
    log.debug('Response for %s: [%d] "%r"', api_path, status_code, body)

    if content_type != 'application/json':
        raise exceptions.NetworkError(f"Invalid response with content type {content_type}: \"{body}\"")

    try:
        result_json = json.loads(body)
    except ValueError:
        result_json = {}

    if status_code == HTTPStatus.OK:
        return result_json

    error_description = result_json.get('errorDescription') or body

    if status_code == HTTPStatus.BAD_REQUEST:
        raise exceptions.BadRequest.detect(error_description)
    if status_code in (HTTPStatus.FORBIDDEN, HTTPStatus.UNAUTHORIZED):
        raise exceptions.Unauthorized.detect(error_description)
    elif status_code == HTTPStatus.TOO_MANY_REQUESTS:
        raise exceptions.RetryAfter
    elif error_description == "webHookUrl timeout":  # most likely the status code of this error is 408, but this is not accurate
        raise exceptions.WebhookUrlError(error_description)

    raise exceptions.MonobankError(f"{error_description} [{status_code}]")


async def make_request(
        session: aiohttp.ClientSession,
        server: MonobankAPIServer,
        http_method: HTTPMethod,
        api_path: str,
        **kwargs
) -> dict:
    """
    The make_request function is a helper function that makes an HTTP request to the server.
    It takes in the following parameters:
        session - The aiohttp ClientSession object used for making requests.
        server - The Server object containing information about the API endpoint and authentication credentials.
        http_method - A string representing which HTTP method to use (e.g., 'GET', 'POST', etc.)  This should be one of
        those defined in constants/HTTP_METHODS, but it can also be any other valid HTTP method if you want to extend
        this library's functionality beyond what is provided by Monobank.

    :param session: aiohttp.ClientSession: Make the request
    :param server: MonobankAPIServer: Get the url of the api endpoint
    :param http_method: HTTPStatus: Specify the http method to use
    :param api_path: str: Log the request and response
    :param **kwargs: Pass a variable number of keyword arguments to the function
    :return: A dictionary
    """
    log.debug('Make request: "%s" with data: "%r"', api_path, kwargs.get('json', {}))

    url = server.api_url(api_path=api_path)

    try:
        async with session.request(http_method, url, **kwargs) as response:
            return check_result(api_path, response.content_type, response.status, await response.text())
    except aiohttp.ClientError as e:
        raise exceptions.NetworkError(f"aiohttp client throws an error: {e.__class__.__name__}: {e}")



from typing import Optional

from pydantic import BaseModel, validator


class Jar(BaseModel):
    """Банка клієнта"""
    id: str
    """Ідентифікатор банки"""
    send_id: str
    """Ідентифікатор для сервісу https://send.monobank.ua/{sendId}"""
    title: str
    """Назва банки"""
    description: str
    """Опис банки"""
    currency_code: int
    """Код валюти банки відповідно ISO 4217"""
    balance: float
    """Баланс банки"""
    goal: Optional[float]
    """Цільова сума для накопичення в банці"""

    @validator('balance', 'goal', pre=True, allow_reuse=True)
    def _convert_from_integer_to_currency_sum(cls, value: int) -> Optional[float]:
        if value is None:
            return value

        return value / 100

    class Config:
        fields = {
            'send_id': 'sendId',
            'currency_code': 'currencyCode',
        }



from pydantic import BaseModel

from .statement_item import Statement


class WebhookStatementData(BaseModel):
    account_id: str
    """Ідентифікатор рахунку"""
    statement: Statement
    """Транзакція"""

    class Config:
        fields = {
            'account_id': 'account',
            'statement': 'statementItem',
        }


class WebhookData(BaseModel):
    type: str
    """Тип даних"""
    data: WebhookStatementData



"""
 - MonobankError
    - ValidationError
    - BadRequest
        - InvalidAccount
        - PeriodError
    - Unauthorized
        - InvalidToken
    - RetryAfter
    - WebhookUrlError
    - NetworkError
"""


class MonobankError(Exception):
    def __init__(self, message=None):
        super(MonobankError, self).__init__(message)


class _MatchErrorMixin:
    match = ''
    text = None

    __subclasses = []

    def __init_subclass__(cls, **kwargs):
        super(_MatchErrorMixin, cls).__init_subclass__(**kwargs)
        # cls.match = cls.match.lower() if cls.match else ''
        if not hasattr(cls, f"_{cls.__name__}__group"):
            cls.__subclasses.append(cls)

    @classmethod
    def check(cls, message) -> bool:
        """
        Compare pattern with message

        :param message: always must be in lowercase
        :return: bool
        """
        return cls.match.lower() in message

    @classmethod
    def detect(cls, description):
        description = description.lower()
        for err in cls.__subclasses:
            if err is cls:
                continue
            if err.check(description):
                raise err(cls.text or description)
        raise cls(description)


class ValidationError(MonobankError):
    pass


class BadRequest(MonobankError, _MatchErrorMixin):
    __group = True


class InvalidAccount(BadRequest):
    match = "invalid account"
    text = "Invalid account"


class PeriodError(BadRequest):
    match = "Period must be no more than 31 days"


class Unauthorized(MonobankError, _MatchErrorMixin):
    __group = True


class InvalidToken(Unauthorized):
    match = 'Unknown \'X-Token\''
    text = 'Invalid token or it has been revoked. ' \
           'Get a new token on the page https://api.monobank.ua'


class RetryAfter(MonobankError):
    def __init__(self, retry_after=60):
        super(RetryAfter, self).__init__(f"Flood control exceeded. Retry in {retry_after} seconds.")
        self.timeout = retry_after


class WebhookUrlError(MonobankError):
    pass


class NetworkError(MonobankError):
    pass



import logging
from http import HTTPStatus

from aiohttp import web
from aiomonobank import MonoPersonal, types

WEBHOOK_HOST = "https://example.com"
WEBAPP_HOST = "localhost"
WEBAPP_PORT = 8822
MONOBANK_API_TOKEN = 'your_token'  # TODO don't forget to replace with your real token

logger = logging.getLogger(__name__)

mono_client = MonoPersonal(MONOBANK_API_TOKEN)

router = web.RouteTableDef()


@router.get("/{token}")
async def check_webhook_url(request: web.Request, token: str) -> web.Response:
    """
    The check_webhook_url function is used to check the webhook URL.

    :param request: web.Request: Get the request object
    :param token: str: Check the token from the webhook url
    :return: A response with the status code 200
    """
    if token == MONOBANK_API_TOKEN:
        logger.debug(f"Successful check webhook URL path: {request.path}")

        return web.json_response(status=HTTPStatus.OK)

    return web.json_response(status=HTTPStatus.FORBIDDEN)


@router.post("/{token}")
async def new_transaction(request: web.Request, token: str) -> web.Response:
    """
    The new_transaction function is a webhook handler for the new transaction event.
    It receives a request from the Monobank API and returns an HTTP response with status code 200.
    The function also logs information about the new transaction.

    :param request: web.Request: Receive the request data
    :param token: str: Check the authenticity of the request
    :return: The status code 200
    """
    webhook_data = types.WebhookData(
        **await request.json()
    )

    if webhook_data.type == "StatementItem":
        logger.debug(f"The account ID of the new transaction: {webhook_data.data.account_id}. "
                     f"Sum: {webhook_data.data.statement.amount} UAH")

        print(webhook_data)

    return web.json_response(status=HTTPStatus.OK)


async def on_startup(app: web.Application):
    result = await mono_client.set_webhook(f"{WEBHOOK_HOST}/{MONOBANK_API_TOKEN}")
    logger.debug(f'Wet webhook result: {result}')


async def on_shutdown(app: web.Application):
    await mono_client.close()


def main() -> None:
    """
    The main function is the entry point for this application.
    It creates an instance of a web.Application object, and then calls run_app() to start the server listening on port 8080.
    The app's startup and shutdown hooks are also registered here.

    :return: None, so it will not be used by the server
    """
    try:
        app = web.Application()

        app.on_startup.append(on_startup)
        app.on_shutdown.append(on_shutdown)

        logger.warning('Start...')

        web.run_app(app, host=WEBAPP_HOST, port=WEBAPP_PORT)
    finally:
        logger.warning('Closed...')


if __name__ == '__main__':
    main()



from enum import StrEnum
from typing import Optional

from pydantic import BaseModel, validator


class AccountTypeEnum(StrEnum):
    BLACK = "black"
    WHITE = "white"
    PLATINUM = "platinum"
    IRON = "iron"
    FOP = "fop"
    YELLOW = "yellow"
    EAID = "eAid"


class CashbackType(StrEnum):
    NONE = "None"
    UAH = "UAH"
    MILES = "Miles"


class Account(BaseModel):
    """Рахунок клієнта"""
    id: str
    """Ідентифікатор рахунку"""
    send_id: str
    """Ідентифікатор для сервісу https://send.monobank.ua/{sendId}"""
    balance: float
    """Баланс рахунку в мінімальних одиницях валюти (копійках, центах)"""
    credit_limit: float
    """Кредитний ліміт"""
    type: AccountTypeEnum
    """Тип рахунку"""
    currency_code: int
    """Код валюти рахунку відповідно ISO 4217"""
    cashback_type: Optional[CashbackType]
    """Тип кешбеку який нараховується на рахунок"""
    masked_pan: list[str]
    """Перелік замаскованих номерів карт (більше одного може бути у преміальних карт)"""
    iban: str
    """IBAN рахунку"""

    @validator('balance', 'credit_limit', pre=True, allow_reuse=True)
    def _convert_from_integer_to_currency_sum(cls, value: int):
        return value / 100

    class Config:
        fields = {
            'send_id': 'sendId',
            'credit_limit': 'creditLimit',
            'currency_code': 'currencyCode',
            'cashback_type': 'cashbackType',
            'masked_pan': 'maskedPan',
        }



from datetime import datetime, timedelta, timezone
from typing import Optional
from http import HTTPMethod  # noqa

from aiocache import cached, Cache
from aiocache.serializers import PickleSerializer

from .api import MonobankAPIServer, MONOBANK_PRODUCTION
from .base import BaseMonobank
from .types import Statement, Client, Currency


class MonoPublic(BaseMonobank):
    """
    Загальна інформація що надається без авторизації.

    Джерело: https://api.monobank.ua/docs/#tag/Publichni-dani
    """
    def __init__(self, connections_limit: Optional[int] = None,
                 server: MonobankAPIServer = MONOBANK_PRODUCTION, **kwargs) -> None:
        super().__init__(
            token=kwargs.get('token', ''),
            validate_token=kwargs.get('validate_token', False),
            connections_limit=connections_limit,
            server=server
        )

    @cached(ttl=300, cache=Cache.MEMORY, serializer=PickleSerializer())
    async def get_currency(self) -> list[Currency]:
        """
        Отримання курсів валют:
            Отримати базовий перелік курсів валют monobank.
            Інформація кешується та оновлюється не частіше 1 разу на 5 хвилин.

        Джерело: https://api.monobank.ua/docs/#tag/Publichni-dani/paths/~1bank~1currency/get

        :return: A list of currency objects
        """
        currency = await self.request(
            HTTPMethod.GET,
            "/bank/currency"
        )

        return [Currency(**cur) for cur in currency]


class MonoPersonal(MonoPublic):
    """
    Інформація, що надається тільки за наявністю token-а доступу, який клієнт може отримати
    в особистому кабінеті https://api.monobank.ua/

    Джерело: https://api.monobank.ua/docs/#tag/Kliyentski-personalni-dani
    """
    def __init__(
            self,
            token: str,
            validate_token: Optional[bool] = True,
            connections_limit: Optional[int] = None,
            server: MonobankAPIServer = MONOBANK_PRODUCTION
    ) -> None:
        """
        Create Monobank API token from https://api.monobank.ua/

        :param token: str: token from https://api.monobank.ua/
        :param server: MonobankAPIServer: Monobank API Server endpoint.
        :raise aiomonobank.utils.exceptions.ValidationError: when the token is invalid
        """
        super().__init__(
            token=token,
            validate_token=validate_token,
            connections_limit=connections_limit,
            server=server
        )

    async def set_webhook(self, webhook_url: str) -> bool:
        """
        Встановлення URL користувача:
            Для підтвердження коректності наданої адреси, на неї надсилається GET-запит.
            Сервер має відповісти строго HTTP статус-кодом 200, і ніяким іншим.
            Якщо валідацію пройдено, на задану адресу починають надсилатися POST запити з подіями.

        Події надсилаються у наступному вигляді:
            POST запит на задану адресу у форматі {type:"StatementItem", data:{account:"...", statementItem:{#StatementItem}}}.
            Якщо сервіс користувача не відповість протягом 5с на команду, сервіс повторить спробу ще через 60 та 600 секунд.
            Якщо на третю спробу відповідь отримана не буде, функція буде вимкнута.
            Відповідь сервера має строго містити HTTP статус-код 200.

        :param webhook_url: str:
        :raise aiomonobank.utils.exceptions.RetryAfter: якщо запити частіше 1 разу в хвилину

        Джерело: https://api.monobank.ua/docs/#tag/Kliyentski-personalni-dani/paths/~1personal~1webhook/post
        """
        await self.request(
            HTTPMethod.POST,
            "/personal/webhook",
            json={"webHookUrl": webhook_url}
        )

        return True

    async def get_client_info(self) -> Client:
        """
        Інформація про клієнта:
            Отримання інформації про клієнта та переліку його рахунків і банок.

        Обмеження на використання функції не частіше ніж 1 раз у 60 секунд.

        Джерело: https://api.monobank.ua/docs/#tag/Kliyentski-personalni-dani/paths/~1personal~1client-info/get

        :raise aiomonobank.utils.exceptions.RetryAfter: якщо запити частіше 1 разу в хвилину
        """
        client = await self.request(
            HTTPMethod.GET,
            "/personal/client-info"
        )

        return Client(**client)

    async def get_statement(self,
                            account_id: str = '0',
                            from_datetime: datetime = None,
                            to_datetime: datetime = None) -> list[Statement]:
        """
        Виписка:
            Отримання виписки за час від {from_datetime} до {to_datetime} часу в секундах у форматі UTC time.
            Максимальний час, за який можливо отримати виписку — 31 доба + 1 година

        Обмеження на використання функції — не частіше ніж 1 раз на 60 секунд.

        Джерело: https://api.monobank.ua/docs/#tag/Kliyentski-personalni-dani/paths/~1personal~1statement~1{account}~1{from}~1{to}/get

        :param account_id: str: Ідентифікатор рахунку або банки з переліків Statement list або 0 - дефолтний рахунок.
        :param from_datetime: datetime: Початок часу виписки.
        :param to_datetime: datetime: Останній час виписки (якщо відсутній, буде використовуватись поточний час).
        :raise aiomonobank.utils.exceptions.InvalidAccount: якщо некоректний account_id
        :raise aiomonobank.utils.exceptions.PeriodError: якщо період більше 31 дня та 1 години
        :raise aiomonobank.utils.exceptions.RetryAfter: якщо запити частіше 1 разу в хвилину
        """
        from_datetime = await timestamp(
            from_datetime or datetime.utcnow() - timedelta(days=31, hours=1)
        )
        to_datetime = await timestamp(
            to_datetime or datetime.utcnow()
        )

        path_params = f'/{account_id}/{from_datetime}/{to_datetime}'

        statements = await self.request(
            HTTPMethod.GET,
            "/personal/statement" + path_params
        )

        return [Statement(**statement) for statement in statements]


async def timestamp(date_time: datetime) -> int:
    """
    The timestamp function takes a datetime object and returns the number of seconds since the Unix epoch.

    :param date_time: datetime: Specify the datetime object that will be passed to the function
    :return: The number of seconds since the epoch
    """
    return int(date_time.replace(tzinfo=timezone.utc).timestamp())
